@@ -1,16 +1,13 @@
 from contextlib import closing
-from io import StringIO
-import http.client
 import json
 import urllib.request
 import urllib.parse
 import urllib.error
 from typing import Dict
-import logging
-import socket
 
 from .un_gzip_stream import un_gzip_stream
 from codalab.common import URLOPEN_TIMEOUT_SECONDS, urlopen_with_retry
+from codalab.worker.upload_util import upload_with_chunked_encoding
 
 
 class RestClientException(Exception):
@@ -127,65 +124,21 @@ class RestClient(object):
         download if False and resumes it if True. If i's not specified the download
         runs to completion
         """
-        CHUNK_SIZE = 16 * 1024
-        TIMEOUT = 60
-        # Start the request.
-        parsed_base_url = urllib.parse.urlparse(self._base_url)
-        path = url + '?' + urllib.parse.urlencode(query_params)
-        if parsed_base_url.scheme == 'http':
-            conn = http.client.HTTPConnection(parsed_base_url.netloc, timeout=TIMEOUT)
-        else:
-            conn = http.client.HTTPSConnection(parsed_base_url.netloc, timeout=TIMEOUT)
-        with closing(conn):
-            conn.putrequest(method, parsed_base_url.path + path)
 
-            # Set headers.
-            headers = {
-                'Authorization': 'Bearer ' + self._get_access_token(),
-                'Transfer-Encoding': 'chunked',
-                'X-Requested-With': 'XMLHttpRequest',
-            }
-            headers.update(self._extra_headers)
-            for header_name, header_value in headers.items():
-                conn.putheader(header_name, header_value)
-            conn.endheaders()
+        headers = {
+            'Authorization': 'Bearer ' + self._get_access_token(),
+            'Transfer-Encoding': 'chunked',
+            'X-Requested-With': 'XMLHttpRequest',
+        }
+        headers.update(self._extra_headers)
 
-            # Use chunked transfer encoding to send the data through.
-            bytes_uploaded = 0
-            while True:
-                to_send = fileobj.read(CHUNK_SIZE)
-                if not to_send:
-                    break
-                conn.send(b'%X\r\n%s\r\n' % (len(to_send), to_send))
-                bytes_uploaded += len(to_send)
-                if progress_callback is not None:
-                    should_resume = progress_callback(bytes_uploaded)
-                    if not should_resume:
-                        raise Exception('Upload aborted by client')
-            conn.send(b'0\r\n\r\n')
-
-            # Read the response.
-            logging.debug("About to read the response... url: %s", url)
-
-            # Sometimes, it may take a while for the server to process
-            # the data and send the response. In this case, we want to
-            # periodically keep sending empty bytes so that the
-            # connection doesn't drop before the response is available.
-            got_response = False
-            while not got_response:
-                try:
-                    response = conn.getresponse()
-                    got_response = True
-                except socket.timeout:
-                    logging.debug("Socket timeout, retrying url: %s", url)
-                    conn.send(b'\0')
-            logging.debug("Finished reading the response, url: %s", url)
-            if response.status != 200:
-                # Low-level httplib module doesn't throw HTTPError
-                raise urllib.error.HTTPError(
-                    self._base_url + path,
-                    response.status,
-                    response.reason,
-                    dict(response.getheaders()),
-                    StringIO(response.read().decode()),
-                )
+        upload_with_chunked_encoding(
+            method=method,
+            base_url=self._base_url,
+            headers=headers,
+            query_params=query_params,
+            fileobj=fileobj,
+            need_response=True,
+            url=url,
+            progress_callback=progress_callback,
+        )
